package fr.inra.pappso.class_pep;

import java.util.Vector;
import java.io.File;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import java.util.Hashtable;
import java.io.Serializable;

import fr.inra.pappso.pep_exception.PepException;
import fr.inra.pappso.sax_parse.msms_output;

public class Ms_run implements Serializable {

	private static final long serialVersionUID = 2L;

	private Vector<Spectre> spectra;

	private Hashtable<String, Spectre> scan_to_spectre;

	private Hashtable<String, Pep_result> scan_to_pepresult;

	private Vector<Pep_result> pepnovo_result;

	private Vector<Fasts_result> fasts_result;

	private Vector<seq_prot> xtandem_prot;

	private File msrun;

	public Ms_run(File F) {
		this.initialise();
		this.msrun = F;
	}

	public Ms_run() {
		this.initialise();
	}

	private void initialise() {
		spectra = new Vector<Spectre>();
		scan_to_spectre = new Hashtable<String, Spectre>();
		pepnovo_result = new Vector<Pep_result>();
		fasts_result = new Vector<Fasts_result>();
		scan_to_pepresult = new Hashtable<String, Pep_result>();
		xtandem_prot = new Vector<seq_prot>();
	}

	public void add_xtandem_prot(seq_prot prot) {
		this.xtandem_prot.add(prot);
	}

	public Vector<seq_prot> get_xtandem_prot() {
		return (this.xtandem_prot);
	}

	public boolean is_protein_exist(seq_prot prot) {
		for (int i = 0; i < xtandem_prot.size(); i++) {
			if (xtandem_prot.elementAt(i).get_accession()
					.equals(prot.get_accession()))
				return (true);
		}
		return (false);
	}

	public void add_spectre(Spectre S) {
		this.spectra.add(S);
		scan_to_spectre.put("" + S.get_scan(), S);
	}

	synchronized public void add_Pepnovo_result(Pep_result pep) {
		this.pepnovo_result.add(pep);
		this.scan_to_pepresult.put(pep.get_Scan(), pep);
	}

	public Vector<Pep_result> get_Pepnovo_result() {
		return pepnovo_result;
	}

	public Spectre get_spectra_to_scan(String scan) throws PepException {
		if (scan_to_spectre.containsKey(scan))
			return (scan_to_spectre.get(scan));
		else {
			throw new PepException(
					"There is no scan in this run corresponding to " + scan);
		}
	}

	public Pep_result get_pepnovo_to_scan(String scan) throws PepException {
		if (scan_to_pepresult.containsKey(scan))
			return (scan_to_pepresult.get(scan));
		else {
			throw new PepException(
					"There is no pepnovo result in this run corresponding to scan "
							+ scan);
		}
	}

	public Spectre get_spectre_to_scan(String scan) throws PepException {
		if (scan_to_spectre.containsKey(scan))
			return (scan_to_spectre.get(scan));
		else {
			throw new PepException(
					"There is no spectra in this run corresponding to scan "
							+ scan);
		}
	}

	public void filter_pepnovo_sequence(float seuil) {
		Vector<Pep_result> temp = new Vector<Pep_result>();
		for (int i = 0; i < pepnovo_result.size(); i++) {
			if (pepnovo_result.elementAt(i).get_PnvScr() > seuil) {
				temp.add(pepnovo_result.elementAt(i));
			}
		}

		pepnovo_result.clear();
		scan_to_pepresult.clear();
		for (int i = 0; i < temp.size(); i++) {
			this.add_Pepnovo_result(temp.elementAt(i));
		}
	}

	public void print_pepnovo_sequence_filter(msms_output out) throws Exception {
		out.write(">" + "Sample_" + msrun.getName()
				+ System.getProperty("line.separator"));
		int count = 0;
		for (int i = 0; i < pepnovo_result.size(); i++) {
			if (!pepnovo_result.elementAt(i).is_use_by_fasts()) {
				if (count != 0)
					out.write("," + System.getProperty("line.separator"));
				out.write(pepnovo_result.elementAt(i).get_Sequence());
				count++;
			}
		}
		if (count == 0)
			throw new PepException("no sequence");
	}

	public boolean contain_pepnovo_sequence_filter() throws Exception {
		int count = 0;
		for (int i = 0; i < pepnovo_result.size(); i++) {
			if (!pepnovo_result.elementAt(i).is_use_by_fasts()) {
				count++;
			}
		}
		if (count == 0)
			return (false);
		return (true);
	}

	public Hashtable<String, Spectre> get_scan_to_spectre() {
		return scan_to_spectre;
	}

	public void set_msrun(File F) {
		this.msrun = F;
	}

	public File get_msrun() {
		return (this.msrun);
	}

	public void add_Fasts_result(Fasts_result result) {
		this.fasts_result.add(result);
	}

	public Vector<Fasts_result> get_Fasts_result() {
		return fasts_result;
	}

	public int get_spectra_number() {
		return (this.spectra.size());
	}

	synchronized public int get_spectra_valid_number(float seuil) {
		int count = 0;
		for (int i = 0; i < spectra.size(); i++) {
			if ((spectra.elementAt(i).get_quality_score() > seuil)
					& (!spectra.elementAt(i).is_Identified()))
				count++;
		}
		return (count);
	}

	public void print_mgf(msms_output out) throws Exception {
		for (int i = 0; i < spectra.size(); i++) {
			out.write(spectra.elementAt(i).get_spectra_in_mgf());
		}
	}

	public void print_mgf_filter_to(msms_output out, float seuil)
			throws Exception {
		for (int i = 0; i < spectra.size(); i++) {
			if ((spectra.elementAt(i).get_quality_score() > seuil)
					& (!spectra.elementAt(i).is_Identified()))
				out.write(spectra.elementAt(i).get_spectra_in_mgf());
		}
	}

	public void serialize(ObjectOutputStream out_ser) throws Exception {
		for (int i = 0; i < spectra.size(); i++) {
			out_ser.writeObject(spectra.elementAt(i));
		}
		for (int j = 0; j < pepnovo_result.size(); j++) {
			out_ser.writeObject(pepnovo_result.elementAt(j));
		}
		for (int z = 0; z < xtandem_prot.size(); z++) {
			out_ser.writeObject(xtandem_prot.elementAt(z));
		}
		for (int k = 0; k < fasts_result.size(); k++) {
			out_ser.writeObject(fasts_result.elementAt(k));
		}
	}

	public void unserialize(ObjectInputStream in) throws Exception {
		Spectre sp;
		Pep_result pep;
		seq_prot prot;
		Fasts_result fast;

		int count_spectra = spectra.size();
		spectra.clear();
		scan_to_spectre.clear();
		for (int i = 0; i < count_spectra; i++) {
			sp = (Spectre) in.readObject();
			this.add_spectre(sp);
		}

		int count_pepresult = pepnovo_result.size();
		pepnovo_result.clear();
		scan_to_pepresult.clear();
		for (int j = 0; j < count_pepresult; j++) {
			pep = (Pep_result) in.readObject();
			this.add_Pepnovo_result(pep);
		}

		int count_xtandem_prot = xtandem_prot.size();
		xtandem_prot.clear();
		for (int z = 0; z < count_xtandem_prot; z++) {
			prot = (seq_prot) in.readObject();
			this.add_xtandem_prot(prot);
		}

		int count_fastsresult = fasts_result.size();
		fasts_result.clear();
		for (int k = 0; k < count_fastsresult; k++) {
			fast = (Fasts_result) in.readObject();
			this.add_Fasts_result(fast);
		}

	}

	public seq_prot get_seq_prot_to_desc(String desc) {
		for (int z = 0; z < xtandem_prot.size(); z++) {
			if (xtandem_prot.elementAt(z).is_same_protein(desc))
				return xtandem_prot.elementAt(z);
		}
		return new seq_prot();
	}

	public void print_prot_result(msms_output out2) throws Exception {
		// TODO Auto-generated method stub
		Fasts_result fas;
		out2.write(this.msrun.getName() + System.getProperty("line.separator"));
		out2.write("Number\tDescription\tEvalue\tPeptides\tDatabases\tNumber\tPeptides"
				+ System.getProperty("line.separator"));
		for (int i = 0; i < this.fasts_result.size(); i++) {
			fas = fasts_result.elementAt(i);
			seq_prot prot = this.get_seq_prot_to_desc(fas.getDescription());
			if (!fas.isContaminant()) {
				StringBuilder S = new StringBuilder("");
				S.append(i + 1 + "\t" + fas.getDescription() + "\t"
						+ fas.getEvalue() + "\t" + fas.getscans().size() + "\t");
				String sample = this.msrun.getName().replaceAll("mzXML",
						"fasta");
				if (fas.getDatabase().equalsIgnoreCase(sample))
					S.append("X!Tandem");
				else
					S.append(fas.getDatabase());
				S.append("\t\"" + prot.getNumber() + "\"\t\""
						+ prot.getPeptides() + "\"");
				S.append(System.getProperty("line.separator"));
				out2.write(S.toString());
			}
		}
		out2.write(System.getProperty("line.separator"));
	}

	public void print_pep_result(msms_output out2) throws Exception {
		// TODO Auto-generated method stub
		Fasts_result fas;
		out2.write(this.msrun.getName() + System.getProperty("line.separator"));
		out2.write("Number" + "\t" + "Description"
				+ System.getProperty("line.separator"));
		out2.write("\tScan\tSequence\tCharge\tMH+theo\tMH+obs\tDeltaMH+\tN-gap\tC-gap\tSequence score\tFilter score"
				+ System.getProperty("line.separator"));
		for (int i = 0; i < this.fasts_result.size(); i++) {
			fas = fasts_result.elementAt(i);
			if (!fas.isContaminant()) {
				String S = i + 1 + "\t" + fas.getDescription()
						+ System.getProperty("line.separator");
				out2.write(S);
				Pep_result pep;
				Spectre spect;
				StringBuilder pep_result = new StringBuilder();
				Vector<String> scans = fas.getscans();
				for (int j = 0; j < scans.size(); j++) {
					pep = scan_to_pepresult.get(scans.elementAt(j));
					spect = this.scan_to_spectre.get(scans.elementAt(j));
					pep_result.append("\t" + spect.get_scan() + "\t");
					pep_result.append(pep.get_Sequence() + "\t");
					pep_result.append(pep.get_Charge() + "\t");

					ions test = new ions();
					test.setSequence(pep.get_Sequence_with_ncter());
					test.setMods(pep.get_modifs());

					pep_result.append(test.get_MH() + "\t");
					float mh = (float) (spect.get_mz() * pep.get_Charge() - 1.00794 * (pep
							.get_Charge() - 1));
					pep_result.append(mh + "\t");
					if ((pep.ge_tN_Gap() == 0) & (pep.get_C_Gap() == 0))
						pep_result.append((float) test.get_MH() - mh);

					else
						pep_result.append("ND");
					pep_result.append("\t" + pep.ge_tN_Gap() + "\t");
					pep_result.append(pep.get_C_Gap() + "\t");
					pep_result.append(pep.get_PnvScr() + "\t");
					pep_result.append(spect.get_quality_score()
							+ System.getProperty("line.separator"));
				}
				out2.write(pep_result.toString());
			}
		}
		out2.write(System.getProperty("line.separator"));
	}

	public void print_align_result(msms_output out2) throws Exception {
		// TODO Auto-generated method stub
		Fasts_result fas;
		out2.write(this.msrun.getName() + System.getProperty("line.separator"));
		for (int i = 0; i < this.fasts_result.size(); i++) {
			fas = fasts_result.elementAt(i);
			if (!fas.isContaminant()) {
				String S = fas.getResults()
						+ System.getProperty("line.separator");
				out2.write(S);
			}
		}
		out2.write(System.getProperty("line.separator"));
	}

	public void print_fasta_protein(msms_output out2) throws Exception {
		// TODO Auto-generated method stub
		for (int i = 0; i < this.xtandem_prot.size(); i++) {
			out2.write(xtandem_prot.elementAt(i).get_fasta());
		}
	}

}
