package fr.inra.pappso.class_pep;

import java.io.File;
import java.io.Serializable;
import java.util.Vector;

import org.apache.log4j.Logger;

import fr.inra.pappso.pep_exception.PepException;

public class Spectre implements Serializable {
	private static final Logger logger = Logger.getLogger(Spectre.class);

	private static final long serialVersionUID = 3L;

	private float RT;
	private float mz;
	private int z;
	private int scan;
	private File msrun;
	private Vector<Float> mass;
	private Vector<Float> intensity;
	private int peak;

	private float quality_score;
	private boolean identified;

	// constructeur
	public Spectre() {
		this.set_quality_score(1);
		this.set_Identified(false);
		mass = new Vector<Float>();
		intensity = new Vector<Float>();
		identified = false;
		quality_score = 0;
	}

	public Vector<Float> get_mass_listes() {
		return (this.mass);
	}

	public Vector<Float> get_intensity_liste() {
		return (this.intensity);
	}

	public int get_scan() {
		return (this.scan);
	}

	public File get_msrun() {
		return (this.msrun);
	}

	public float get_mz() {
		return (this.mz);
	}

	public float get_RT() {
		return (this.RT);
	}

	public void set_msrun(File F) {
		this.msrun = F;
	}

	public void set_RT(float F) {
		this.RT = F;
	}

	public void set_mz(float F) {
		this.mz = F;
	}

	public void set_charge(int I) {
		this.z = I;
	}

	public void set_scan(int I) {
		this.scan = I;
	}

	public void set_peak(int I) {
		this.peak = I;
	}

	public void set_spectra(float[] liste) throws PepException {
		// transformer le spectre String en Vector mass/intensity
		for (int i = 0; i < liste.length; i += 2) {
			mass.add(liste[i]);
			intensity.add(liste[i + 1]);
		}
		if (mass.size() != peak)
			throw new PepException("Error decoding mass for" + scan + " "
					+ peak + " != " + mass.size());
		if (intensity.size() != peak)
			throw new PepException("Error decoding intensity for" + scan + " "
					+ peak + " != " + intensity.size());
	}

	public void set_quality_score(float quality_score) {
		this.quality_score = quality_score;
	}

	public float get_quality_score() {
		return quality_score;
	}

	public void set_Identified(boolean identified) {
		this.identified = identified;
	}

	public boolean is_Identified() {
		return identified;
	}

	public void print_spectra() {
		for (int i = 0; i < mass.size(); i++) {
			logger.debug(mass.elementAt(i) + " " + intensity.elementAt(i));
		}
		logger.debug("ok");
	}

	public String get_spectra_in_mgf() {
		String mgf = "";
		mgf += "BEGIN IONS" + System.getProperty("line.separator");
		mgf += "TITLE=" + msrun.getName() + "_" + scan
				+ System.getProperty("line.separator");
		mgf += "PEPMASS=" + mz + System.getProperty("line.separator");
		mgf += "SCANS=" + scan + System.getProperty("line.separator");
		mgf += "RTINSECONDS=" + RT + System.getProperty("line.separator");
		if (z != 0) {
			mgf += "CHARGE=" + z + "+" + System.getProperty("line.separator");
		}

		for (int i = 0; i < mass.size(); i++) {
			mgf += mass.elementAt(i) + " " + intensity.elementAt(i)
					+ System.getProperty("line.separator");
		}

		mgf += "END IONS" + "\n\n";

		return (mgf);
	}

}
