package fr.inra.pappso.gui;

import javax.swing.*;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;

import org.apache.log4j.Logger;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URI;
import java.util.Arrays;
import java.util.Vector;

import fr.inra.pappso.class_pep.Ms_run;
import fr.inra.pappso.pep_exception.PepException;
import fr.inra.pappso.pep_exception.StopException;
import fr.inra.pappso.pipeline.export_pepnovo_sequence;
import fr.inra.pappso.pipeline.fasts_conf;
import fr.inra.pappso.pipeline.pepnovo_conf;
import fr.inra.pappso.pipeline.process;
import fr.inra.pappso.sax_parse.Serialising;
import fr.inra.pappso.sax_parse.msms_output;

public class main extends JFrame implements ActionListener {
	private static final Logger logger = Logger.getLogger(main.class);

	// A deplacer dans le main une fois l'arborescence
	// fr.inra.pappso.denovopipeline créee
	static public String version = "1.5.1";

	static public String news = "V1.5.1 :\n" + "* Java code refactoring\n"
			+ "* PepNovo model selection";

	private static final long serialVersionUID = 12L;

	private JTextPane action_label = new JTextPane();

	private JFileChooser fileChooser;

	private my_file_filter FT;

	private pepnovo_conf pepconf;

	private fasts_conf fastconf;

	private String repertoire_in;

	private String repertoire_out;

	private Vector<view_result> results;

	private Dimension size = new Dimension(600, 500);

	public main() {
		super("De Novo pipeline v" + main.version);
		try {
			// Creation de la fenetre principale (boutons...)
			JFrame_initialise();
			// mettre la fenetre visible
			setVisible(true);

		} catch (Exception e) {
			System.err.println("" + e);
			System.exit(1);
		}
	}

	private void JFrame_initialise() throws Exception {
		repertoire_in = new String("");
		repertoire_out = new String("");
		results = new Vector<view_result>();

		// Creation des menus
		JMenuBar menuBar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");
		menuBar.add(fileMenu);
		JMenu process = new JMenu("Processing");
		menuBar.add(process);
		JMenu result = new JMenu("Result");
		menuBar.add(result);
		JMenu about = new JMenu("About");
		menuBar.add(about);

		// creation des sous-menu
		JMenuItem Action1 = new JMenuItem("DeNovo sequencing");
		JMenuItem Action2 = new JMenuItem("Homology search");
		JMenuItem close = new JMenuItem("Close");
		JMenuItem Action4 = new JMenuItem("Export");
		JMenuItem Action3 = new JMenuItem("View result");
		JMenuItem doc = new JMenuItem("Documentation");
		JMenuItem news = new JMenuItem("News");

		// Ajout des sous-menu au menu
		process.add(Action1);
		process.add(Action2);
		result.add(Action3);
		result.add(Action4);
		fileMenu.add(close);
		about.add(doc);
		about.add(news);

		// creation des actions sur les menus
		Action1.addActionListener(this);
		Action2.addActionListener(this);
		Action3.addActionListener(this);
		Action4.addActionListener(this);
		close.addActionListener(this);
		doc.addActionListener(this);
		news.addActionListener(this);

		// remplissage du texte
		StringBuilder S = new StringBuilder();
		S.append("<center><b>This program is a pipeline performing automatic DeNovo identification</b><br />on mzXML files coming from ion trap mass spectrometers.</center>");
		S.append("<p>It uses two modules : Pepnovo and Fasts.</p>");
		S.append("<p>It filters spectra previously identified by X!Tandem software (_pepnovo.xml) and works only on the remaining ones.</p>");
		S.append("<p><b>DeNovo sequencing : </b> Every MS/MS spectra, of good quality, is interpreted to obtain the best possible sequence using the PepNovo software (version 3.1).</p>");
		S.append("<p><b>Homology search : </b> Every good interpretation from DeNovo sequencing is submitted to homology search using Fasts software (version 36.2.6) in an iterative process.</p>");
		S.append("<p><b>View result :</b> Graphical display of the homology search result and of the interpreted MS/MS spectra;</p>");
		S.append("<p><b>Export :</b> Possibility to export results in various formats.</p>");

		action_label.setContentType("text/html");
		action_label.setText(S.toString());
		action_label.setEditable(false);

		// initialisation de la fenetre
		// getContentPane().add(panel, BorderLayout.SOUTH);
		getContentPane()
				.add(new JScrollPane(action_label), BorderLayout.CENTER);
		setSize(650, 400);
		setJMenuBar(menuBar);

		// initialisation de l'operation de fermeture de la fenetre
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// centrage de la fenetre
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation(dim.width / 2 - getWidth() / 2, dim.height / 2
				- getHeight() / 2);

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {

			UIManager.setLookAndFeel(UIManager
					.getCrossPlatformLookAndFeelClassName());
			MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme());
			UIManager.setLookAndFeel(new MetalLookAndFeel());
		}
	}

	private File[] get_files_mzxml() throws StopException {
		fileChooser = new JFileChooser();
		fileChooser.setDialogTitle("Select the mzXML files to be analyzed");
		// fileChooser.setFileSelectionMode(JFileChooser.OPEN_DIALOG);
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.setMultiSelectionEnabled(true);
		fileChooser.setCurrentDirectory(new File(repertoire_in));
		FT = new my_file_filter(new String[] { "mzXML", "mzxml" },
				"mzXML file (*.mzXML)");
		fileChooser.setFileFilter(FT);
		fileChooser.setPreferredSize(size);
		// fileChooser.setFileFilter(new FileFilter(".xml"));
		int returnValue = fileChooser.showOpenDialog(this);
		if (returnValue == JFileChooser.CANCEL_OPTION)
			throw new StopException("No mzMXL file selected");
		File[] files = fileChooser.getSelectedFiles();
		repertoire_in = "" + fileChooser.getCurrentDirectory();
		Arrays.sort(files);
		return (files);
	}

	private File[] get_files_pepnovo() throws StopException {
		fileChooser = new JFileChooser();
		fileChooser
				.setDialogTitle("Select the DeNovo result files to be analyzed");
		// fileChooser.setFileSelectionMode(JFileChooser.OPEN_DIALOG);
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.setMultiSelectionEnabled(true);
		fileChooser.setCurrentDirectory(new File(repertoire_in));
		FT = new my_file_filter(new String[] { "pepnovo" },
				"Pepnovo file (*.pepnovo)");
		fileChooser.setFileFilter(FT);
		fileChooser.setPreferredSize(size);
		// fileChooser.setFileFilter(new FileFilter(".xml"));
		int returnValue = fileChooser.showOpenDialog(this);
		if (returnValue == JFileChooser.CANCEL_OPTION)
			throw new StopException("No pepnovo file selected");
		File[] files = fileChooser.getSelectedFiles();
		repertoire_in = "" + fileChooser.getCurrentDirectory();
		Arrays.sort(files);
		return (files);
	}

	// private File[] get_files_fasts() throws StopException {
	// fileChooser = new JFileChooser();
	// fileChooser.setDialogTitle("Select the Fasts result files to analyzed");
	// fileChooser.setFileSelectionMode(JFileChooser.OPEN_DIALOG);
	// fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
	// fileChooser.setMultiSelectionEnabled(true);
	// fileChooser.setCurrentDirectory(new File(repertoire_in));
	// FT = new my_file_filter(new String[] { "fasts" },
	// "Fasts file (*.fasts)");
	// fileChooser.addChoosableFileFilter(FT);
	// // fileChooser.setFileFilter(new FileFilter(".xml"));
	// int returnValue = fileChooser.showOpenDialog(this);
	// if (returnValue == JFileChooser.CANCEL_OPTION)
	// throw new StopException("No pepnovo file selected");
	// File[] files = fileChooser.getSelectedFiles();
	// repertoire_in = "" + fileChooser.getCurrentDirectory();
	// Arrays.sort(files);
	// return (files);
	// }

	private File get_file_fasta() throws StopException {
		fileChooser = new JFileChooser();
		fileChooser
				.setDialogTitle("Select the fasta database file to be analyzed");
		// fileChooser.setFileSelectionMode(JFileChooser.OPEN_DIALOG);
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.setMultiSelectionEnabled(false);
		fileChooser.setCurrentDirectory(new File(repertoire_in));
		FT = new my_file_filter(new String[] { "fasta" },
				"Fasta file (*.fasta)");
		fileChooser.setFileFilter(FT);
		fileChooser.setPreferredSize(size);
		// fileChooser.setFileFilter(new FileFilter(".xml"));
		int returnValue = fileChooser.showOpenDialog(this);
		if (returnValue == JFileChooser.CANCEL_OPTION)
			throw new StopException("No fasta file selected");
		File file = fileChooser.getSelectedFile();
		repertoire_in = "" + fileChooser.getCurrentDirectory();
		return (file);
	}

	private File get_file_save() throws StopException {
		fileChooser = new JFileChooser();
		fileChooser.setDialogTitle("Save the results on file");
		// fileChooser.setFileSelectionMode(JFileChooser.SAVE_DIALOG);
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.setMultiSelectionEnabled(false);
		if (repertoire_out.equalsIgnoreCase(""))
			fileChooser.setCurrentDirectory(new File(repertoire_in));
		else {
			fileChooser.setCurrentDirectory(new File(repertoire_out));
		}
		int returnValue = fileChooser.showSaveDialog(this);
		if (returnValue == JFileChooser.CANCEL_OPTION)
			throw new StopException("No saving file selected");
		File file_save = fileChooser.getSelectedFile();
		repertoire_out = "" + fileChooser.getCurrentDirectory();
		return (file_save);
	}

	private File get_dir_save() throws StopException {
		fileChooser = new JFileChooser();
		fileChooser.setDialogTitle("Save the results on folder");
		fileChooser.setFileSelectionMode(JFileChooser.SAVE_DIALOG);
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		fileChooser.setMultiSelectionEnabled(false);
		if (repertoire_out.equalsIgnoreCase(""))
			fileChooser.setCurrentDirectory(new File(repertoire_in));
		else {
			fileChooser.setCurrentDirectory(new File(repertoire_out));
		}
		int returnValue = fileChooser.showSaveDialog(this);
		if (returnValue == JFileChooser.CANCEL_OPTION)
			throw new StopException("No saving folder selected");
		File file_save = fileChooser.getSelectedFile();
		repertoire_out = "" + fileChooser.getCurrentDirectory();
		return (file_save);
	}

	private void fasts_clic(ActionEvent evt) {
		try {
			fastconf = new Serialising().unserialise_fasts_conf();

			configuration_windows config = new configuration_windows(this,
					true, fastconf);
			if (config.isStop())
				throw new StopException("");

			File[] files = this.get_files_pepnovo();
			fastconf.setDatabase_interest(this.get_file_fasta());
			fastconf.setDataout(get_file_save());

			new running_fasts(this, true, fastconf, files);

			// ajouter l'ecriture des résultats

			msms_output prot = new msms_output();
			prot.open(fastconf.getDataout().getAbsolutePath() + "_prot.txt");
			msms_output pept = new msms_output();
			pept.open(fastconf.getDataout().getAbsolutePath() + "_peptide.txt");
			msms_output align = new msms_output();
			align.open(fastconf.getDataout().getAbsolutePath()
					+ "_alignment.txt");

			for (int i = 0; i < files.length; i++) {
				File fastresult = new File(fastconf.getOutdir()
						.getAbsolutePath()
						+ System.getProperty("file.separator")
						+ files[i].getName().replaceAll(".pepnovo", "")
						+ "."
						+ "fasts");

				if (fastresult.exists()) {
					Ms_run run = new Serialising().unserialise_msrun(fastresult
							.getAbsolutePath());
					run.print_prot_result(prot);
					run.print_align_result(align);
					run.print_pep_result(pept);

				}
			}
			prot.close();
			align.close();
			pept.close();

			JOptionPane.showMessageDialog(this,
					"Analysis finished, the results are saved on :\n"
							+ fastconf.getDataout().getAbsolutePath() + "*\n",
					"Message", JOptionPane.INFORMATION_MESSAGE);

		} catch (StopException e) {
			JOptionPane.showMessageDialog(this, "Analysis stopped\n", "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (Exception e) {
			StackTraceElement[] f = e.getStackTrace();
			String error = e.getMessage() + "\nat : " + f[0].toString();
			JOptionPane.showMessageDialog(this, new JScrollPane(new JTextArea(
					"Problems during the analysis\n" + error)), "Error",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}
	}

	private void pepnovo_clic(ActionEvent evt) {

		try {
			pepconf = new Serialising().unserialise_pep_conf();

			configuration_windows config = new configuration_windows(this,
					true, pepconf);
			if (config.isStop())
				throw new StopException("");

			File[] files = this.get_files_mzxml();
			pepconf.setOut_dir(this.get_dir_save());
			new running_pepnovo(this, true, pepconf, files);

			JOptionPane.showMessageDialog(this,
					"Analysis finished, the results are saved on :\n"
							+ pepconf.getOut_dir().getAbsolutePath() + "\n",
					"Message", JOptionPane.INFORMATION_MESSAGE);
		} catch (StopException e) {
			JOptionPane.showMessageDialog(this, "Analysis stopped\n", "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (Exception e) {
			StackTraceElement[] f = e.getStackTrace();
			String error = e.getMessage() + "\nat : " + f[0].toString();
			JOptionPane.showMessageDialog(this, new JScrollPane(new JTextArea(
					"Problems during the analysis\n" + error)), "Error",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}
	}

	private void view_clic(ActionEvent evt) {
		try {
			results.add(new view_result());
		} catch (Exception e) {
			StackTraceElement[] f = e.getStackTrace();
			String error = e.getMessage() + "\nat : " + f[0].toString();
			JOptionPane.showMessageDialog(this, new JScrollPane(new JTextArea(
					"Problems during the analysis\n" + error)), "Error",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}
	}

	private void export_clic(ActionEvent evt) {
		JComboBox export_choice = new JComboBox();
		export_pepnovo_sequence export1 = new export_pepnovo_sequence();
		export_choice.addItem(export1);
		JOptionPane.showMessageDialog(this, export_choice, "Select export",
				JOptionPane.PLAIN_MESSAGE, null);
		try {
			process pro = (process) export_choice.getSelectedItem();
			if (pro.equals(export1)) {
				pro.set_files_to_analysed(this.get_files_pepnovo());
				pro.setSave(this.get_file_save());
			}
			new running(this, true, pro);

			if (!pro.getError().equalsIgnoreCase(""))
				throw new PepException(pro.getError());
			if (pro.isStop())
				throw new StopException("");

			JOptionPane.showMessageDialog(this,
					"Export finished, the results are saved as :\n"
							+ pro.getSave().getAbsolutePath() + "\n",
					"Message", JOptionPane.INFORMATION_MESSAGE);
		} catch (StopException e) {
			JOptionPane.showMessageDialog(this, "Analysis stopped\n", "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (Exception e) {
			StackTraceElement[] f = e.getStackTrace();
			String error = e.getMessage() + "\nat : " + f[0].toString();
			JOptionPane.showMessageDialog(this, new JScrollPane(new JTextArea(
					"Problems during the analysis\n" + error)), "Error",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}

		// new JScrollPane(new JTextArea(
		// "This fonction are under construction\nBe patient ;)"))
	}

	public void actionPerformed(ActionEvent evt) {
		String composant = evt.getActionCommand();
		if (composant.equalsIgnoreCase("DeNovo sequencing"))
			pepnovo_clic(evt);
		else if (composant.equalsIgnoreCase("Homology search"))
			fasts_clic(evt);
		else if (composant.equalsIgnoreCase("View result"))
			view_clic(evt);
		else if (composant.equalsIgnoreCase("Export"))
			export_clic(evt);
		else if (composant.equalsIgnoreCase("Close")) {
			for (int i = 0; i < results.size(); i++) {
				results.elementAt(i).dispose();
			}
			dispose();
		} else if (composant.equalsIgnoreCase("Documentation")) {
			String u = "http://pappso.inra.fr/downloads/DeNovo_pipeline.pdf";
			if (Desktop.isDesktopSupported()) {
				// On récupère l'instance du desktop :
				Desktop desktop = Desktop.getDesktop();
				if (desktop.isSupported(Desktop.Action.BROWSE)) {
					// Et on lance l'application associée au protocole :
					try {
						desktop.browse(new URI(u));
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		} else if (composant.equalsIgnoreCase("News")) {
			JOptionPane.showMessageDialog(this, main.news, "News",
					JOptionPane.INFORMATION_MESSAGE);
		}
	}

}
