package fr.inra.pappso.pipeline;

import java.io.File;

import org.apache.log4j.Logger;

import fr.inra.pappso.class_pep.Ms_run;
import fr.inra.pappso.pep_exception.PepException;
import fr.inra.pappso.pep_exception.StopException;
import fr.inra.pappso.sax_parse.Serialising;
import fr.inra.pappso.sax_parse.msms_output;
import fr.inra.pappso.sax_parse.mzxml_parse;
import fr.inra.pappso.sax_parse.parse_file;
import fr.inra.pappso.sax_parse.tandem_to_pepnovo_parse;
import fr.inra.pappso.shell.pepnovo_calculation;
import fr.inra.pappso.shell.pepnovo_filter;

public class pepnovo_pip extends Thread {
	private static final Logger logger = Logger.getLogger(pepnovo_pip.class);

	File mzxml;

	String basename;

	Ms_run run;

	pepnovo_conf conf;

	String status;

	String error;

	boolean stoping;

	public pepnovo_pip() {
		super();
		logger.debug("pepnovo_pip begin");
		this.setStatus("Initialising");
		this.setDaemon(true);
		stoping = false;
		// System.getProperty("file.separator");
		// System.getProperty("os.name");
		run = new Ms_run();
		error = "";
	}

	public void run() {
		try {
			// parsing
			logger.debug("pepnovo_pip run begin");
			this.setStatus("Parsing");
			mzxml_parse handler = new mzxml_parse(mzxml);
			new parse_file(handler, mzxml);
			Ms_run run = handler.get_msruns();
			logger.debug("parsing ok");

			// creation du mgf
			this.setStatus("Mgf creation");
			msms_output out = new msms_output();
			out.open(this.get_out("mgf"));
			run.print_mgf(out);
			out.close();

			// quality filter
			pepnovo_filter filter = new pepnovo_filter();
			filter.set_commande(this.get_commande_filter());
			filter.set_Msrun(run);
			filter.start_process();
			this.setStatus("Quality filter");
			while (filter.status()) {
				if (this.stoping) {
					filter.stop_process();
					throw new StopException("");
				}
				Thread.sleep(1000);
			}
			if (!filter.get_error().equals(""))
				throw new PepException(filter.get_error());
			logger.debug("filter");

			// xtandem parsing
			File xtandem = conf.getXmlFile();
			if (xtandem != null && xtandem.exists()) {
				this.setStatus("Xtandem result filter");
				tandem_to_pepnovo_parse xtand_handler = new tandem_to_pepnovo_parse(
						run, basename);
				new parse_file(xtand_handler, xtandem);
				logger.debug("xtandem_filter");
			}
			// File xtandem = new File(conf.getXml_dir()
			// + System.getProperty("file.separator") + basename + ".xml");
			// if (xtandem.exists()) {
			// status = "Xtandem result filter";
			// xtandem_parse xtand_handler = new xtandem_parse(run);
			// new parse_file(xtand_handler, xtandem);
			// System.out.println("xtandem_filter");
			// }

			// creation du mgf
			this.setStatus("Mgf creation");
			msms_output out2 = new msms_output();
			out2.open(this.get_out("mgf"));
			run.print_mgf_filter_to(out2, conf.getSeuil());
			out2.close();

			// Pepnovo calculation
			logger.debug("pepnovo_calculation");
			this.setStatus("Pepnovo calculation");
			pepnovo_calculation pep = new pepnovo_calculation();
			pep.set_Msrun(run);
			pep.set_seuil_filter(conf.getSeuil());
			pep.set_commande(this.get_commande_calculation());
			pep.start_process();
			while (pep.status()) {
				if (this.isStoping()) {
					pep.stop_process();
					throw new StopException("");
				}
				Thread.sleep(1000);
				this.setStatus("Pepnovo : " + pep.get_count() + ":"
						+ run.get_spectra_valid_number(conf.getSeuil()));
			}
			if (!pep.get_error().equals("")) {
				String message = pep.get_error();
				logger.error(message);
				throw new PepException(message);
			}

			// sérialisation
			this.setStatus("Save serialising");
			logger.debug("Save serialising");
			new Serialising().serialise_msrun(this.get_out("pepnovo"), run);
			logger.debug("serialise");
			logger.debug("finish");

		} catch (StopException e) {
			logger.debug("Stop");
		} catch (Exception e) {
			this.setError("" + e);
			e.printStackTrace();
		} finally {
			File mgf = new File(this.get_out("mgf"));
			if (mgf.exists())
				mgf.delete();
		}

	}

	synchronized private boolean isStoping() {
		return this.stoping;
	}

	private String get_out(String extension) {
		return (conf.getOut_dir().getAbsolutePath()
				+ System.getProperty("file.separator") + basename + "." + extension);
	}

	private String[] get_commande_filter() {
		// TODO Auto-generated method stub
		String[] com = { conf.getPepnovo_exec().getAbsolutePath(), "-model",
				conf.getPepnovoModel(), "-file", this.get_out("mgf"), "-PTMs",
				"C+57:M+16", "-model_dir",
				conf.getModel_dir().getAbsolutePath(), "-pmcsqs_only" };// -sqs_only
		return com;

	}

	// "CID_IT_TRYP"
	private String[] get_commande_calculation() {
		String[] com = { conf.getPepnovo_exec().getAbsolutePath(), "-model",
				conf.getPepnovoModel(), "-file", this.get_out("mgf"), "-PTMs",
				"C+57:M+16", "-model_dir",
				conf.getModel_dir().getAbsolutePath(), "-num_solutions", "1",
				"-correct_pm", "-no_quality_filter" };
		return com;

	}

	public pepnovo_conf getConf() {
		return conf;
	}

	public void setConf(pepnovo_conf conf) {
		this.conf = conf;
	}

	public String getError() {
		return error;
	}

	private void setError(String error) {
		this.error = error;
	}

	public File getMzxml() {
		return mzxml;
	}

	public void setMzxml(File mzxml) {
		this.mzxml = mzxml;
		this.basename = this.mzxml.getName().replaceAll(".mzXML", "");
	}

	public Ms_run getRun() {
		return run;
	}

	public void setRun(Ms_run run) {
		this.run = run;
	}

	synchronized public String getStatus() {
		return status;
	}

	synchronized public String setStatus(String status) {
		return this.status = status;
	}

	synchronized public void stop_process() {
		stoping = true;
	}

}
